import { ISqlite } from '../interfaces';
/**
 * Allows for using strings and `sql-template-strings`. Converts both to a
 * format that's usable by the SQL methods
 *
 * @param sql A SQL string or `sql-template-strings` object
 * @param params An array of parameters
 */
export declare function toSqlParams(sql: ISqlite.SqlType, params?: any[]): ISqlite.SqlObj;
